@@ -1,4 +1,4 @@
-import types
+import types, re
 from datetime import date, time, datetime
 
 from django.utils.safestring import mark_safe
@@ -219,6 +219,10 @@ class ModelList(ModelInfoBase):
         group_template = '<tr><td colspan="%(cols)s" class="group"><h3>%(display)s</h3></td></tr>'
         groups = []
         show_header = True
+        show_summary = True
+        summary_fields = None
+        summary_td_template = '<td>%(val)s</td>'
+        summary_tr_template = '<tr>%(cells)s</tr>'
 
     def __init__(self, queryset, *args, **kwargs):
         self.queryset = queryset
@@ -232,7 +236,8 @@ class ModelList(ModelInfoBase):
         for attr in ('model','fields','exclude','td_template','th_template','tr_template',
                 'thead_template','tbody_template','show_if_none','auto_urlize',
                 'auto_linebreaks','list_display_links','icon_edit_template',
-                'icon_delete_template','group_template','groups','show_header'):
+                'icon_delete_template','group_template','groups','show_header',
+                'summary_fields','summary_td_template','summary_tr_template','show_summary'):
             if not hasattr(self._meta, attr):
                 setattr(self._meta, attr, getattr(_meta, attr))
 
@@ -240,12 +245,14 @@ class ModelList(ModelInfoBase):
         self._meta.td_template = self._meta.td_template.replace(r'%s', r'%(value)s')
 
     def render_buttons_header(self):
+        """Renders the header cell for the column that will have util buttons"""
         if self._meta.icon_edit_template or self._meta.icon_delete_template:
             return '<th class="buttons">&nbsp;</th>'
         else:
             return ''
 
     def render_buttons_cell(self, instance, edit_url=None, delete_url=None, additional_code=''):
+        """Renders the cell with util buttons for each row in the grid"""
         ret = []
 
         url = hasattr(instance, 'get_absolute_url') and getattr(instance, 'get_absolute_url')() or ''
@@ -264,10 +271,10 @@ class ModelList(ModelInfoBase):
             return ''
 
     def header(self):
+        """Renders a header with field labels on the top of the grid"""
         fields = self._meta.fields or self.get_model_fields()
 
         # THead
-
         thead = []
 
         for f_name in fields:
@@ -278,7 +285,48 @@ class ModelList(ModelInfoBase):
 
         return self._meta.thead_template %(''.join(thead) + self.render_buttons_header())
 
+    def summary(self):
+        """Renders a summary with aggregation in the end of the grid.
+        
+        Uses the meta attributes:
+            
+         * show_summary
+         * summary_fields
+         * summary_td_template
+         * summary_tr_template
+        """
+        fields = self._meta.fields or self.get_model_fields()
+
+        try:
+            qs = self.queryset.all()
+        except AttributeError:
+            qs = self.queryset
+
+        tsummary = []
+
+        for f_name in fields:
+            if f_name in self._meta.summary_fields:
+                val = self._meta.summary_fields[f_name](qs)
+            else:
+                val = '&nbsp'
+            
+            tsummary.append(self._meta.summary_td_template%{
+                'val': val,
+                })
+
+        return self._meta.summary_tr_template%{
+                'cells': ''.join(tsummary) + self.render_button_cell_summary(),
+                }
+
+    def render_button_cell_summary(self):
+        return '<td>&nbsp;</td>'
+
+    def get_columns_count(self):
+        fields = self._meta.fields or self.get_model_fields()
+        return len(fields)+1
+
     def rows(self):
+        """Iterator with the list of rows in the grid"""
         fields = self._meta.fields or self.get_model_fields()
 
         tbody = []
@@ -310,35 +358,18 @@ class ModelList(ModelInfoBase):
                     field_value = self.get_field_display_value(group, obj)
                     if groups_values[group] != field_value:
                         groups_values[group] = field_value
-                        group_row = self._meta.group_template%{'cols': len(fields)+1, 'display': field_value}
+                        group_row = self._meta.group_template%{'cols': self.get_columns_count(), 'display': field_value}
                         group_rows.append(group_row)
 
             # Single row
             row = self.render_single_object(obj)
             ret = self.item_class(self, '\n'.join(group_rows) + row, obj)
 
-            """
-            row = []
-
-            for i, f_name in enumerate(fields):
-                f_value = self.get_field_display_value(f_name, obj)
-                f_value = self.get_linkable_field_value(
-                        obj, f_name, f_value,
-                        force=not self._meta.list_display_links and i == 0,
-                        )
-
-                if self._meta.show_if_none or f_value is not None:
-                    row.append(self._meta.td_template%{'field_name': f_name, 'value': f_value})
-                else:
-                    row.append(self._meta.td_template%{'field_name': f_name, 'value': '&nbsp;'})
-
-            ret = self.item_class(self, '\n'.join(group_rows) + self._meta.tr_template%(
-                        ''.join(row) + self.render_buttons_cell(obj)), obj)
-            """
-            
             yield ret
 
     def render_single_object(self, obj, tr_template=None, td_template=None):
+        """Renders a single object. It is util internally or can be used for
+        granulary customizations"""
         # Customized templates
         tr_template = tr_template or self._meta.tr_template
         td_template = td_template or self._meta.td_template
@@ -363,6 +394,7 @@ class ModelList(ModelInfoBase):
         return ret
 
     def as_string(self):
+        """Returns the whole string with the grid HTML code"""
         fields = self._meta.fields or self.get_model_fields()
 
         # THead
@@ -380,5 +412,9 @@ class ModelList(ModelInfoBase):
         else:
             ret = u''
 
+        # Summary
+        if self._meta.show_summary and self._meta.summary_fields:
+            ret += self.summary()
+
         return mark_safe(ret)
 
