@@ -453,11 +453,6 @@ register.tag('remove_from_get_params', do_remove_from_get_params)
 
 # Model info
 
-#@register.simple_tag
-#def model_info_for_object(class_path, object):
-#    cls = path_to_object(class_path)
-#    return cls(object)
-
 class ModelInfoForObjectNode(template.Node):
     class_path = None
     obj = None
@@ -494,22 +489,17 @@ def do_model_info_for_object(parser, token):
 register.tag('model_info_for_object', do_model_info_for_object)
 
 
-#@register.simple_tag
-#def model_info_for_list(class_path, list):
-#    cls = path_to_object(class_path)
-#    return cls(list)
-
 class ModelInfoForListNode(template.Node):
     class_path = None
     list_obj = None
-    as_part = None
     as_varname = None
+    piece = None
 
-    def __init__(self, class_path, list_obj, as_part=None, as_varname=None):
+    def __init__(self, class_path, list_obj, as_varname=None, piece=None):
         self.class_path = template.Variable(class_path)
         self.list_obj = template.Variable(list_obj)
-        self.as_part = as_part
         self.as_varname = as_varname
+        self.piece = piece
 
     def render(self, context):
         class_path = self.class_path.resolve(context)
@@ -518,7 +508,13 @@ class ModelInfoForListNode(template.Node):
         cls = path_to_object(class_path)
         model_list = cls(list_obj, request=context.get('request', None))
 
-        if not self.as_part:
+        if self.piece:
+            model_list = getattr(model_list, self.piece, None)
+
+            if callable(model_list):
+                model_list = model_list()
+
+        if not self.as_varname:
             return model_list
 
         context[self.as_varname] = model_list
@@ -527,10 +523,24 @@ class ModelInfoForListNode(template.Node):
 def do_model_info_for_list(parser, token):
     try:
         parts = token.split_contents()
+
+        class_path = parts[1]
+        list_obj = parts[2]
+
+        if 'as' in parts:
+            as_varname = parts[parts.index('as')+1]
+        else:
+            as_varname = None
+            
+        if 'only' in parts:
+            piece = parts[parts.index('only')+1]
+        else:
+            piece = None
+
     except ValueError:
         raise template.TemplateSyntaxError, "You must inform class path and list object"
 
-    return ModelInfoForListNode(*parts[1:])
+    return ModelInfoForListNode(class_path, list_obj, as_varname, piece)
 
 register.tag('model_info_for_list', do_model_info_for_list)
 
